@@ -1,4 +1,4 @@
-#' Merging of satellite datasets with ground observations using Random Forest
+#' @title Merging of satellite datasets with ground observations using Random Forest
 #'
 #' @description
 #' RFmerge is a methodology developed by Baez-Villanueva et al. (2020) for the fusion of satellite precipitation datasets with ground-based observations, with the objective of improving the accuracy and spatial representativeness of the data.
@@ -25,7 +25,7 @@
 #'   - Each station column contains numeric values representing observed measurements.
 #'   - The column names (station identifiers) must be unique and match those in `BD_Coord$Cod` to ensure proper spatial referencing.
 #' @param BD_Coord A `data.table` or `data.frame` containing the metadata of the ground stations. It must include the following columns:
-#' #' - \code{"Cod"}:
+#' - \code{"Cod"}:
 #'    Unique identifier for each ground station.
 #'
 #' - \code{"X"}:
@@ -36,9 +36,9 @@
 #'
 #' - \code{"Z"}:
 #'   Altitude of the station in meters.
-#' @param cov A list of covariates used as independent variables in the RFmerge. Each covariate should be a
+#' @param cov A list of cov used as independent variables in the RFmerge. Each covariate should be a
 #'   `SpatRaster` object (from the `terra` package) and can represent satellite-derived weather variables or a Digital
-#'    Elevation Model (DEM). All covariates should have the same number of layers (bands), except for the DEM, which must have only one layer.
+#'    Elevation Model (DEM). All cov should have the same number of layers (bands), except for the DEM, which must have only one layer.
 #'
 #' @param mask A shapefile defining the study area.
 #'   If provided, the shapefile must be of class `SpatVector` (from the `terra` package) with a UTM coordinate reference system.
@@ -60,17 +60,17 @@
 #'  Each list item should represent a category, with the category name as the list item name and a numeric vector specifying the lower and upper bounds of that category.
 #'  \strong{Note:} See the "Notes" section for additional details on how to define categories, use this parameter for validation, and example configurations.
 #' @param save_model Logical value indicating whether the interpolation file should be saved to disk. The default value is `FALSE`. indicating that the interpolated file should not be saved.
-#     If set to `TRUE`, be sure to set the working directory beforehand using `setwd(path)` to specify where the files should be saved.
+#'     If set to `TRUE`, be sure to set the working directory beforehand using `setwd(path)` to specify where the files should be saved.
 #' @param name_save Character string indicating the name under which the interpolation raster file will be saved. By default the algorithm sets as output name: 'Model_RFmerge'.
 #' as the code will internally assign it.
 #' @references Baez-Villanueva, O. M.; Zambrano-Bigiarini, M.; Beck, H.; McNamara, I.; Ribbe, L.; Nauditt, A.; Birkel, C.; Verbist, K.; Giraldo-Osorio, J.D.; Thinh, N.X. (2020). RF-MEP: a novel Random Forest method for merging gridded precipitation products and ground-based measurements, Remote Sensing of Environment, 239, 111610. doi:10.1016/j.rse.2019.111606.
 #' @examples
 #' \donttest{
-#' library(InterpolateR)
 #' # Load data from on-site observations
 #'  data("BD_Obs", package = "InterpolateR")
 #'  data("BD_Coord", package = "InterpolateR")
-#' # Load the covariates
+#'
+#' # Load the cov
 #' cov <- list(
 #'  MSWEP = terra::rast(system.file("extdata/MSWEP.nc", package = "InterpolateR")),
 #'  CHIRPS = terra::rast(system.file("extdata/CHIRPS.nc", package = "InterpolateR")),
@@ -83,7 +83,6 @@
 #'                          Rain_threshold = NULL, save_model = FALSE, name_save = NULL)
 #'
 #' # Visualize the results
-#'
 #' # Precipitation results within the study area
 #' modelo_rainfall = model_RFmerge$Ensamble
 #'
@@ -93,7 +92,6 @@
 #'
 #' # categorical metrics
 #' metrics_cat = model_RFmerge$Validation$categorical_metrics
-#'
 #' }
 #' @return If a value other than 1 is set (point to pixel validation is performed), a list containing two elemeentis returned:
 #'
@@ -118,7 +116,7 @@
 #'   The parameter should be entered as a named list, where each item represents a category and the name of the item is the category name.
 #'   The elements of each category must be a numeric vector with two values: the lower and upper limits of the category.
 #'   For example:
-#' #' \code{Rain_threshold = list(
+#' \code{Rain_threshold = list(
 #'   no_rain = c(0, 1),
 #'   light_rain = c(1, 5),
 #'   moderate_rain = c(5, 20),
@@ -133,7 +131,6 @@
 RFmerge = function(BD_Obs, BD_Coord, cov, mask = NULL, n_round = NULL, ntree = 2000,
                    seed = 123,  training = 1, stat_validation = NULL, Rain_threshold = NULL,
                    save_model = FALSE, name_save = NULL) {
-
   ##############################################################################
   #                               Check input data                             #
   ##############################################################################
@@ -156,7 +153,7 @@ RFmerge = function(BD_Obs, BD_Coord, cov, mask = NULL, n_round = NULL, ntree = 2
   if (!inherits(BD_Coord, c("data.table", "data.frame"))) stop("BD_Coord must be a 'data.table' or a 'data.frame'.")
 
   # Check that the coordinate names appear in the observed data
-  if (!all(BD_Coord$Cod %chin% setdiff(names(BD_Obs), "Date"))) stop("The names of the coordinates do not appear in the observed data.")
+  if (!all(BD_Coord$Cod %in% base::setdiff(names(BD_Obs), "Date"))) stop("The names of the coordinates do not appear in the observed data.")
 
   # Check if mask is a SpatVector object
   if (!is.null(mask) && !inherits(mask, "SpatVector")) stop("mask must be a 'SpatVector' object.")
@@ -164,9 +161,6 @@ RFmerge = function(BD_Obs, BD_Coord, cov, mask = NULL, n_round = NULL, ntree = 2
   # Verify that all dates have at least one entry recorded
   Dates_NA <- BD_Obs[apply(BD_Obs[, .SD, .SDcols = -1], 1, function(x) all(is.na(x))), Date]
   if (length(Dates_NA) > 0) stop(paste0("No data was found for the dates: ", paste(Dates_NA, collapse = ", ")))
-
-  # Check that the coordinate names appear in the observed data
-  if (!all(BD_Coord$Cod %chin% setdiff(names(BD_Obs), "Date"))) stop("The names of the coordinates do not appear in the observed data.")
   ##############################################################################
   #               Verify that there is a DEM and manage DEM layers.            #
   ##############################################################################
@@ -179,18 +173,12 @@ RFmerge = function(BD_Obs, BD_Coord, cov, mask = NULL, n_round = NULL, ntree = 2
   nlyrs_tots <- which(nlyr_covs != 1)
   nlyr_rep <- nlyr_covs[nlyrs_tots[1]]
   DEM <- cov[[index_dem]]
-  DEM <- terra::rast(replicate(nlyr_rep, DEM))
-  cov[[index_dem]] <- DEM
-
-  # Verify the layers of the cov.
-  if (length(unique(sapply(cov, function(x) terra::nlyr(x)))) > 1) stop("The number of covariate layers does not match. Check the input data.")
-
   ##############################################################################
   #                          Verify if validation is to be done                #
   ##############################################################################
   if (training != 1 | !is.null(stat_validation)) {
     data_val = .select_data(BD_Obs, BD_Coord, training = training,
-                           seed = seed, stat_validation = stat_validation)
+                            stat_validation = stat_validation)
     train_data = data_val$train_data
     train_cords = data_val$train_cords
   } else {
@@ -205,101 +193,116 @@ RFmerge = function(BD_Obs, BD_Coord, cov, mask = NULL, n_round = NULL, ntree = 2
   Sample_lyrs <- DEM[[1]] * 0
 
   # Data for training
-  training_data <- melt(
+  training_data <- data.table::melt(
     train_data,
     id.vars = "Date",
     variable.name = "Cod",
     value.name = "var"
-  )[, ID := as.numeric(factor(Cod))]
+  )
 
   # Date of the data
   Dates_extracted <- unique(training_data[, Date])
   Points_Train <- merge(training_data, train_cords, by = "Cod")
-  setDT(Points_Train)
+  data.table::setDT(Points_Train)
 
-  Points_Train <- unique(Points_Train, by = "Cod")[, .(ID, Cod, X, Y, Z)]
-  setorder(Points_Train, ID)
-
-  Points_VectTrain <- terra::vect(Points_Train, geom = c("X", "Y"), crs = crs(Sample_lyrs))
+  Points_Train <- unique(Points_Train, by = "Cod")[, .(Cod, X, Y, Z)]
+  Points_VectTrain <- terra::vect(Points_Train, geom = c("X", "Y"), crs = terra::crs(Sample_lyrs))
 
   # Calculate the Distance Euclidean
-  distance_ED <- setNames(lapply(1:nrow(Points_VectTrain), function(i) {
+  distance_ED <- stats::setNames(lapply(1:nrow(Points_VectTrain), function(i) {
     terra::distance(Sample_lyrs, Points_VectTrain[i, ], rasterize = FALSE)
   }), Points_VectTrain$Cod)
   ##############################################################################
   #                                 RF merge method                            #
   ##############################################################################
-  RF_Modelmerge = function(day_COV, fecha) {
-    names(day_COV) = sapply(day_COV, names)
+  day_COV <- list(
+    DEM = DEM,
+    distance_ED = terra::rast(distance_ED)
+  )
 
-    data_obs <- training_data[Date == as.Date(fecha), ]
+  day_COV = terra::rast(day_COV)
+  data_cov = lapply(day_COV, terra::extract, y = Points_VectTrain) |>
+    Reduce(\(x, y) merge(x, y, by = "ID", all = TRUE), x = _) |>
+    (\(d) {
+      data.table::setDT(d)
+    })()
 
-    if (data_obs[, sum(var, na.rm = TRUE)] == 0) return(Sample_lyrs) # if the sum of var is 0, I assume that there is no precipitation in the whole basin.
+  data_cov$DEM <- Points_VectTrain$Z
+  data_cov$Cod <- Points_VectTrain$Cod
+  data_cov$ID = NULL
 
-    # If the sum of var not is 0, I assume that there is precipitation in the whole basin.
-      points_EstTrain <- merge(
-        data_obs[, .(ID, Cod)],
-        Points_Train[, .(Cod, X, Y, Z)],
-        by = "Cod"
-      )[order(ID)] |>
-        terra::vect(geom = c("X", "Y"), crs = crs(Sample_lyrs))
+  # Generate data for prediction
+  name_covs <- names(cov)[nlyr_covs != 1]
+  data_simSat = lapply(name_covs, function (name) {
+    raster = cov[[name]]
+    dt = data.table::data.table(terra::extract(raster, y = Points_VectTrain))
+    dt[, Cod := Points_VectTrain$Cod]
+    dt[,ID := NULL]
+    dt = data.table::transpose(dt, make.names = "Cod")
+    dt = cbind(
+      Date = Dates_extracted,
+      dt
+    )
 
-        add_rasters <- function(lyr, pattern) {
-          r = terra::rast(get(lyr)[points_EstTrain$Cod])  # Obtiene el raster basado en la capa `lyr`
-          names(r) = paste0(pattern, "_", seq_along(points_EstTrain$Cod))  # Usa `pattern` en los nombres
-          r
-        }
+    dt <- data.table::melt(
+      dt,
+      id.vars = "Date",
+      variable.name = "Cod",
+      value.name = name
+    )
+    return(dt)
+  })
 
-        day_COV$dist_ED <- add_rasters("distance_ED", "dist_ED")
+  dt_sim =  Reduce(function(x, y) merge(x, y, by = c("Date", "Cod"), all = TRUE), data_simSat)
+  dt_merged = merge(dt_sim, data_cov, by = "Cod", all.x = TRUE)
+  dt_final = merge(dt_merged, training_data[, .(Date, Cod, var)], by = c("Date", "Cod"), all.x = TRUE)
 
-          data_cov = lapply(day_COV, terra::extract, y = points_EstTrain) |>
-            Reduce(\(x, y) merge(x, y, by = "ID", all = TRUE), x = _) |>
-            (\(d) {
-              setDT(d)
-              d[, DEM := points_EstTrain$Z[match(ID, points_EstTrain$ID)]]
-              d
-            })()
+  # Module for Ffmerge refactoring
 
-            dt.train = merge(
-              data_obs[, .(ID, var)],
-              data_cov,
-              by = "ID"
-            )
-            features <- setdiff(names(dt.train), "ID")
-            cov_Sat <- terra::rast(day_COV)
-                Model_P1 <- randomForest::randomForest(
-                  var ~ .,
-                  data = dt.train[, ..features],
-                  ntree = ntree,
-                  na.action = na.omit
-                ) |>
-                  suppressWarnings()
+  RF_Modelmerge = function(date_P, Cov_pred_combined) {
+    train_RF = dt_final[Date == date_P, ]
+    if (train_RF[, sum(var, na.rm = TRUE)] == 0) return(Sample_lyrs)
+    features_ff <- base::setdiff(names(train_RF), c("Cod", "Date"))
+    set.seed(seed)
 
-                return(terra::predict(cov_Sat, Model_P1, na.rm = TRUE))
+    Model_P1 <- randomForest::randomForest(
+      var ~ .,
+      data = train_RF[, ..features_ff],
+      ntree = ntree,
+      na.action = stats::na.omit # REVISAR ESTO
+     ) |>
+      suppressWarnings()
+
+    return(terra::predict(Cov_pred_combined, Model_P1, na.rm = TRUE))
   }
 
   # Run the model
-  pbapply::pboptions(type = "timer", use_lb = T, style = 1, char = "=")
+  Cov_pred <- cov[!grepl("DEM", names(cov))]
+  pbapply::pboptions(type = "timer", use_lb = F, style = 1, char = "=")
   message("Analysis in progress. Please wait...")
-  raster_Model <- pbapply::pblapply(Dates_extracted, function(fecha) {
-    day_COV <- lapply(cov, function(x) x[[match(fecha, Dates_extracted)]])
-    RF_Modelmerge(day_COV, fecha)
+  raster_Model <- pbapply::pblapply(Dates_extracted, function(date_P) {
+    Cov_pred <- lapply(Cov_pred, function(x) x[[match(date_P, Dates_extracted)]])
+    Cov_pred_combined <- c(Cov_pred, list(day_COV))
+    Cov_pred_combined <- terra::rast(Cov_pred_combined)
+    ff = base::setdiff(names(dt_final), c("Cod", "Date", "var"))
+    names(Cov_pred_combined) = ff
+    RF_Modelmerge(date_P, Cov_pred_combined)
   })
 
   Ensamble <- terra::rast(raster_Model)
+
   if (!is.null(n_round)) Ensamble <- terra::app(Ensamble, \(x) round(x, n_round))
 
   if (!is.null(mask)) {
     Ensamble <- terra::mask(Ensamble, mask)
   }
-
   ##############################################################################
   #                           Perform validation if established                #
   ##############################################################################
   if (training != 1 | !is.null(stat_validation)) {
     test_cords = data_val$test_cords
     test_data = data_val$test_data
-    final_results <- .validate(test_cords, test_data, crss = crs(Sample_lyrs),
+    final_results <- .validate(test_cords, test_data, crss = terra::crs(Sample_lyrs),
                               Ensamble, Rain_threshold = Rain_threshold)
   }
   ##############################################################################
@@ -311,6 +314,9 @@ RFmerge = function(BD_Obs, BD_Coord, cov, mask = NULL, n_round = NULL, ntree = 2
     name_saving <- paste0(name_save, ".nc")
     terra::writeCDF(Ensamble, filename = name_saving, overwrite=TRUE)
   }
+  ##############################################################################
+  #                                      Return                                #
+  ##############################################################################
   if (training != 1 | !is.null(stat_validation)) return(list(Ensamble = Ensamble, Validation = final_results))
   if (training == 1 & is.null(stat_validation)) return(Ensamble)
-}
+} # End Rfmerge function
